@@ -294,17 +294,121 @@ class Strategy52WHigh(Strategy):
             return result
         
 # Option Strategies
-class OptionIVDiffStrategy:
+class OptionIVDiffStrategy(Strategy):
     
-    def sortStrikes(self, up, x):
+    file_path = '{0}_OptionIVDiffStrategy.json'.format(datetime.today().strftime("%Y%m%d"))
+    helper = OptionHelper()
+    
+    def getTickerList(self, pending = False, processed = False):
+
+        if self.ticker_list is None:
+            ticker_list = self.helper.fnoList()
+        else:
+            ticker_list = self.ticker_list
+        processed_tickers = self.saved_data.get('processed_tickers', [])
+        pending_tickers = list(set(ticker_list) - set(processed_tickers))
+        if pending:
+            return pending_tickers
+        if processed:
+            return processed_tickers
+        return ticker_list
+    
+    def processStrategy(self, ticker, nextExpiryCount = 1):
+        
+        df = self.helper.formatedOptionChain(ticker, False)
+        nearestExpiry = list(set(df.expiryDate.values))
+        nearestExpiry.sort()
+        nearestExpiry = nearestExpiry[nextExpiryCount-1]
+        df = df[df['expiryDate'].isin([nearestExpiry])]
+
+        callStrikes = list(set((df[df['iv_ce'] > 0].strikePrice.values)))
+        putStrikes = list(set((df[df['iv_pe'] > 0].strikePrice.values)))
+        strikes = list(set((df.strikePrice.values)))
+        strikes.sort(reverse = False)
+        up = df.underlyingValue.values[0]
+
+        iv_ce = round(np.mean(df[df['strikePrice'].isin(self._getClosestStrikes(callStrikes, up))].iv_ce),2)
+        iv_pe = round(np.mean(df[df['strikePrice'].isin(self._getClosestStrikes(putStrikes, up))].iv_pe),2)
+        iv = round((iv_ce + iv_pe)/2,2)
+        days_to_expiry = (datetime.strptime(nearestExpiry, '%Y-%m-%d') - datetime.today()).days
+        low = round(up*(1 - iv*np.sqrt(days_to_expiry/365)/100),2)
+        high = round(up*(1 + iv*np.sqrt(days_to_expiry/365)/100),2)
+        strike_low = self._getClosestStrikes(putStrikes, low,1)[0]
+        strike_low_nxt = strikes[strikes.index(strike_low) - 1]
+        strike_high = self._getClosestStrikes(callStrikes, high,1)[0]
+        strike_high_nxt = strikes[strikes.index(strike_high) + 1]
+        strike_low_put_price = df[df['strikePrice'].isin([strike_low])].put_price.values[0]
+        strike_low_put_nxt_price = df[df['strikePrice'].isin([strike_low_nxt])].put_price.values[0]
+        strike_high_call_price = df[df['strikePrice'].isin([strike_high])].call_price.values[0]
+        strike_high_call_nxt_price = df[df['strikePrice'].isin([strike_high_nxt])].call_price.values[0]
+        data = {
+            'ticker' : ticker,
+            'iv_ce' : iv_ce,
+            'iv_pe' : iv_pe,
+            'underlying' : float(up),
+            'days_to_expiry' : days_to_expiry,
+            'iv' : iv,
+            'low' : low,
+            'high' : high,
+            'iv_diff' : round(iv_pe - iv_ce,2),
+            'strike_low' : float(strike_low),
+            'strike_high' : float(strike_high),
+            'strike_low_pe_price' : strike_low_put_price,
+            'strike_high_ce_price' : strike_high_call_price,
+            'strike_low_nxt' : float(strike_low_nxt),
+            'strike_high_nxt' : float(strike_high_nxt),
+            'strike_low_pe_nxt_price' : strike_low_put_nxt_price,
+            'strike_high_ce_nxt_price' : strike_high_call_nxt_price
+            }
+        return data
+    
+    def screener(self,nextExpiryCount = 1):
+        
+        result = []
+        ticker_list = self.getTickerList()
+        pending_tickers = self.getTickerList(pending = True)
+        processed_tickers = self.getTickerList(processed = True)
+        
+        if len(pending_tickers) != 0:
+            if self.save:
+                sample_tickers = random.sample(pending_tickers, min(10, len(pending_tickers)))
+            else:
+                sample_tickers = pending_tickers
+            for ticker in sample_tickers:
+                try:
+                    data = self.processStrategy(ticker, nextExpiryCount)
+                    result.append(data)
+                except:
+                    pass
+
+            # Save to file
+            result = self.saved_data.get('result', []) + result
+            processed_tickers = processed_tickers + sample_tickers
+            data = {
+                'processed_tickers': processed_tickers + sample_tickers,
+                'result' : result
+            }
+
+            self.saveToFile(data)
+            if len(processed_tickers) < len(ticker_list):
+                return None
+        else:
+            result = self.saved_data.get('result', [])
+            
+        data = pd.DataFrame(result)
+        data['Action'] = data.apply(lambda x: self._getAction(x), axis = 1)
+        return data
+    
+    
+    def _sortStrikes(self, up, x):
         return abs(up-x)
 
-    def getClosestStrikes(self, strikes, value, count = 3):
-        strikes.sort(key = lambda x: self.sortStrikes(value, x))
+    def _getClosestStrikes(self, strikes, value, count = 3):
+        strikes.sort(key = lambda x: self._sortStrikes(value, x))
         filtered_strikes = strikes[:count]
         return filtered_strikes
 
-    def getAction(self, x, threshold = 10):
+    def _getAction(self, x, threshold = 10):
         action = ''
         if abs(x['iv_diff']) >= threshold:
             if x['iv_diff'] > 0:
@@ -315,62 +419,6 @@ class OptionIVDiffStrategy:
                 action +=  ' and Buy ' + x['ticker'] + ' CALL@' + str(x['strike_high_nxt']) + ' for ' + str(x['strike_high_ce_nxt_price'])
         return action
     
-    def screener(self,nextExpiryCount = 1):
-        data = []
-        failed = []
-        helper = OptionHelper()
-        for ticker in tqdm(helper.fnoList()[:5]):
-            try:
-                df = helper.formatedOptionChain(ticker, False)
-                nearestExpiry = list(set(df.expiryDate.values))
-                nearestExpiry.sort()
-                nearestExpiry = nearestExpiry[nextExpiryCount-1]
-                df = df[df['expiryDate'].isin([nearestExpiry])]
-
-                callStrikes = list(set((df[df['iv_ce'] > 0].strikePrice.values)))
-                putStrikes = list(set((df[df['iv_pe'] > 0].strikePrice.values)))
-                strikes = list(set((df.strikePrice.values)))
-                strikes.sort(reverse = False)
-                up = df.underlyingValue.values[0]
-
-                iv_ce = round(np.mean(df[df['strikePrice'].isin(self.getClosestStrikes(callStrikes, up))].iv_ce),2)
-                iv_pe = round(np.mean(df[df['strikePrice'].isin(self.getClosestStrikes(putStrikes, up))].iv_pe),2)
-                iv = round((iv_ce + iv_pe)/2,2)
-                days_to_expiry = (datetime.strptime(nearestExpiry, '%Y-%m-%d') - datetime.today()).days
-                low = round(up*(1 - iv*np.sqrt(days_to_expiry/365)/100),2)
-                high = round(up*(1 + iv*np.sqrt(days_to_expiry/365)/100),2)
-                strike_low = self.getClosestStrikes(putStrikes, low,1)[0]
-                strike_low_nxt = strikes[strikes.index(strike_low) - 1]
-                strike_high = self.getClosestStrikes(callStrikes, high,1)[0]
-                strike_high_nxt = strikes[strikes.index(strike_high) + 1]
-                strike_low_put_price = df[df['strikePrice'].isin([strike_low])].put_price.values[0]
-                strike_low_put_nxt_price = df[df['strikePrice'].isin([strike_low_nxt])].put_price.values[0]
-                strike_high_call_price = df[df['strikePrice'].isin([strike_high])].call_price.values[0]
-                strike_high_call_nxt_price = df[df['strikePrice'].isin([strike_high_nxt])].call_price.values[0]
-                data.append({
-                    'ticker' : ticker,
-                    'iv_ce' : iv_ce,
-                    'iv_pe' : iv_pe,
-                    'underlying' : up,
-                    'days_to_expiry' : days_to_expiry,
-                    'iv' : iv,
-                    'low' : low,
-                    'high' : high,
-                    'iv_diff' : round(iv_pe - iv_ce,2),
-                    'strike_low' : strike_low,
-                    'strike_high' : strike_high,
-                    'strike_low_pe_price' : strike_low_put_price,
-                    'strike_high_ce_price' : strike_high_call_price,
-                    'strike_low_nxt' : strike_low_nxt,
-                    'strike_high_nxt' : strike_high_nxt,
-                    'strike_low_pe_nxt_price' : strike_low_put_nxt_price,
-                    'strike_high_ce_nxt_price' : strike_high_call_nxt_price
-                    })
-            except:
-                failed.append(ticker)
-        data = pd.DataFrame(data)
-        data['Action'] = data.apply(lambda x: self.getAction(x), axis = 1)
-        return data
  
 # Screener Strategies       
 class ScreenerData:
